SystemOrganization addCategory: #'Rest-Client-HTTP'!
SystemOrganization addCategory: #'Rest-Client-Proxies'!
SystemOrganization addCategory: #'Rest-Client-Prefetch'!
SystemOrganization addCategory: #'Rest-Client'!

SequenceableCollection subclass: #RestProxyCollection
	instanceVariableNames: 'proxyObjects restUri repository loaded'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

RestProxyCollection subclass: #RestPrefetchProxyCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Prefetch'!

!RestPrefetchProxyCollection methodsFor: 'updating' stamp: 'cs 6/11/2009 14:57'!
reload

	self repository monitor: self.
	self isLoaded ifFalse: [super reload].! !

!RestPrefetchProxyCollection methodsFor: 'updating' stamp: 'cs 6/11/2009 14:57'!
reloadNeeded

	self isLoaded ifTrue: [super reload].! !

!RestProxyCollection class methodsFor: 'instance creation' stamp: 'mt 3/31/2009 15:39'!
fromPath: aPathString 

	^ self new restUri: aPathString! !

!RestProxyCollection methodsFor: 'comparing' stamp: 'cs 4/25/2009 16:49'!
= otherCollection

	^ self == otherCollection! !

!RestProxyCollection methodsFor: 'adding' stamp: 'cs 4/26/2009 18:21'!
add: aRestObject 

	| newObject |
	newObject := (aRestObject id isNil or: [ aRestObject id < 1 ]) 
		ifTrue: 
			[ self repository 
				addObject: self restUri
				with: aRestObject ]
		ifFalse: 
			[ self repository 
				addObject: self restUri
				with: aRestObject restUri ].
	self reloadNeeded.
	self changed: (self collectionTypeAsSymbol, #Added) asSymbol with: aRestObject.
	^ newObject reloadMissingReferences! !

!RestProxyCollection methodsFor: 'adding' stamp: 'cs 4/26/2009 18:21'!
addNew: aPropertyDict 

	| newObject |
	newObject := self repository 
		addObject: self restUri
		with: aPropertyDict.
	self reloadNeeded.
	self changed: (self collectionTypeAsSymbol, #Added) asSymbol with: newObject.
	^ newObject reloadMissingReferences! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:46'!
at: index

	self ensureLoaded.
	^ self proxyObjects at: index! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/31/2009 15:46'!
baseUri

	^ self repository url asString! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mfp 4/23/2009 14:13'!
collectionTypeAsSymbol

	^ self restUri ifNotNilDo: [:uri | (uri findTokens: $/) last asSymbol]! !

!RestProxyCollection methodsFor: 'updating' stamp: 'cs 6/11/2009 15:00'!
ensureLoaded

	self isLoaded ifFalse: [self reload].! !

!RestProxyCollection methodsFor: 'adding' stamp: 'mt 4/28/2009 10:19'!
fill: someRestUris
	"Take list of rest uris and create proxy objects to fill this collection."
	
	| newProxyObjects |
	someRestUris isString ifTrue: [self restUri: someRestUris. ^ self].
	newProxyObjects := someRestUris collect: [:path | self repository getObject: path].
	self proxyObjects = newProxyObjects ifTrue: [^ self].
	self
		proxyObjects: newProxyObjects;
		loaded: true;
		changed: (self collectionTypeAsSymbol, #Loaded) asSymbol.! !

!RestProxyCollection methodsFor: 'comparing' stamp: 'cs 4/25/2009 16:49'!
hash

	^ self identityHash! !

!RestProxyCollection methodsFor: 'initialization' stamp: 'mig 4/1/2009 17:00'!
initialize
	
	super initialize.
	self loaded: false.
	self proxyObjects: OrderedCollection new.! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'cs 6/11/2009 14:50'!
isLoaded

	^ loaded! !

!RestProxyCollection methodsFor: 'testing' stamp: 'cs 3/17/2009 18:21'!
isRestProxy

	^ true! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'cs 4/23/2009 10:50'!
loaded: anObject

	loaded := anObject.! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:21'!
proxyObjects

	^ proxyObjects! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'cs 3/17/2009 16:31'!
proxyObjects: anObject

	proxyObjects := anObject.! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'cs 4/22/2009 18:46'!
referencedProxies

	^ proxyObjects! !

!RestProxyCollection methodsFor: 'updating' stamp: 'cs 6/11/2009 14:59'!
referencesProxy: aRestProxy

	^ self ~= aRestProxy
		and: [self isLoaded
			and: [self anySatisfy: [:proxy | proxy = aRestProxy]]]! !

!RestProxyCollection methodsFor: 'updating' stamp: 'cs 4/27/2009 17:50'!
reload

	self reloadDepth: 1.! !

!RestProxyCollection methodsFor: 'updating' stamp: 'SR 7/22/2009 22:12'!
reloadDepth: aNumber

	| response |
	response := RestProxyObject socketClass get: ('{1}{2}{3}' format: {
		self baseUri. self restUri.
		aNumber ~= 1 ifTrue: ['?depth=', aNumber asString] ifFalse: ['']}).
	response isOk
		ifTrue: [self fill: (Json readFrom: response content readStream)]
		ifFalse: [
			response isNotFound 
				ifTrue: [self removeFromCache; changed: #deleted]
				ifFalse: [RestProxyRepository errorForCode: response responseCode]].! !

!RestProxyCollection methodsFor: 'updating' stamp: 'cs 6/11/2009 14:59'!
reloadNeeded

	self isLoaded ifTrue: [self reload].! !

!RestProxyCollection methodsFor: 'removing' stamp: 'mfp 4/23/2009 14:54'!
remove: aRestObject ifAbsent: aBlock

	self repository deleteObject: (self restUri , '/', aRestObject id asString).
	self reloadNeeded.
	self changed: (self collectionTypeAsSymbol, #Removed) asSymbol with: {aRestObject}.
	aRestObject reloadObsoleteReferences.! !

!RestProxyCollection methodsFor: 'deleting' stamp: 'topa 4/6/2009 16:54'!
removeFromCache

	^ self repository removeFromCache: self restUri
! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/16/2009 11:16'!
repository

	^ repository! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/16/2009 11:17'!
repository: anObject 

	repository := anObject.! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:21'!
restUri

	^ restUri! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'cs 3/17/2009 16:32'!
restUri: anObject 

	restUri := anObject.! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:47'!
size

	self ensureLoaded.
	^ self proxyObjects size! !

!RestProxyCollection methodsFor: 'private' stamp: 'mt 3/6/2009 13:03'!
species

	^ Array! !

PluggableDialogWindow subclass: #RestLoginDialog
	instanceVariableNames: 'loginEntry passwordEntry connectButton cancelButton result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestLoginDialog methodsFor: 'controls' stamp: 'SR 7/22/2009 16:20'!
cancel

	self result: false.
	^ super cancel! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'SR 3/19/2009 19:27'!
cancelButton

	cancelButton ifNil: [
		cancelButton := UITheme current 
			newButtonIn: self
			for: self
			getState: nil
			action: #cancel
			arguments: nil
			getEnabled: nil
			label: 'Cancel'
			help: 'Close login window'].
	^ cancelButton! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'topa 3/6/2009 14:50'!
cancelButton: anObject

	cancelButton := anObject.! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'SR 7/22/2009 12:02'!
connectButton

	connectButton ifNil: [
		connectButton := UITheme current 
			newButtonIn: self
			for: self
			getState: nil
			action: #ok
			arguments: nil
			getEnabled: nil
			label: 'Ok'
			help: ''.
		connectButton isDefault: true].
	^ connectButton! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'topa 3/6/2009 14:50'!
connectButton: anObject

	connectButton := anObject.! !

!RestLoginDialog methodsFor: 'user interface' stamp: 'SR 7/22/2009 12:02'!
createCol: someElements

	^ UITheme current newColumnIn: self for: someElements! !

!RestLoginDialog methodsFor: 'user interface' stamp: 'SR 7/22/2009 11:59'!
createLabel: aString

	^ UITheme current
		newLabelIn: self
		label: aString! !

!RestLoginDialog methodsFor: 'user interface' stamp: 'SR 7/22/2009 12:18'!
createLoginEntryFor: myModel

	self loginEntry: ((UITheme current 
		newTextEntryIn: self
		for: myModel
		get: #login
		set: #login:
		class: String
		getEnabled: nil
		help: 'Enter login') maxLength: 100).
	^ self loginEntry minWidth: 200! !

!RestLoginDialog methodsFor: 'user interface' stamp: 'topa 3/6/2009 14:51'!
createPasswordEntryFor: myModel

	self passwordEntry: ((UITheme current 
		newTextEntryIn: self
		for: myModel
		get: #password
		set: #password:
		class: String
		getEnabled: nil
		help: 'Enter password') maxLength: 100).
	self passwordEntry textMorph font: (StrikeFont passwordFontSize: 12).
	^ self passwordEntry! !

!RestLoginDialog methodsFor: 'user interface' stamp: 'SR 7/22/2009 12:02'!
createRow: someElements

	^ UITheme current newRowIn: self for: someElements! !

!RestLoginDialog methodsFor: 'initialization' stamp: 'SR 7/22/2009 12:05'!
initialize

	| myModel |
	super initialize.
	
	myModel := XpfLoginModelForHand new.
	self
		buttons: {self connectButton. self cancelButton};
		contentMorph: (self initializeUserInterface: myModel);
		model: myModel;
		addGrips;
		title: 'Login'.! !

!RestLoginDialog methodsFor: 'initialization' stamp: 'SR 7/22/2009 11:59'!
initializeUserInterface: myModel

	^ self createCol: {
		self createRow: {
			self createLabel: 'Login:'.
			self createLoginEntryFor: myModel}.
		self createRow: {
			self createLabel: 'Password:'.
			self createPasswordEntryFor: myModel}}! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'topa 3/6/2009 14:51'!
loginEntry

	^ loginEntry! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'topa 3/6/2009 14:51'!
loginEntry: anObject

	loginEntry := anObject! !

!RestLoginDialog methodsFor: 'controls' stamp: 'SR 7/22/2009 12:13'!
ok

	self loginEntry accept.
	self passwordEntry accept.
	
	self model validLoginAndPassword
		ifTrue: [
			self result: true.
			self model storeDataToHand: self ownerHand.
			^ super ok]
		ifFalse: [
			self abort: 'Login or password incorrect']! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'topa 3/6/2009 14:52'!
passwordEntry

	^ passwordEntry! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'topa 3/6/2009 14:52'!
passwordEntry: anObject

	passwordEntry := anObject.! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'topa 3/6/2009 14:52'!
result

	^ result! !

!RestLoginDialog methodsFor: 'accessing' stamp: 'topa 3/6/2009 14:52'!
result: anObject

	result := anObject.! !

!HandMorph methodsFor: '*Rest-Client-override' stamp: 'SR 7/22/2009 10:06'!
drawOn: aCanvas 
	"Draw the hand itself (i.e., the cursor)."

	| userPic |
	self temporaryCursor isNil 
		ifTrue: [aCanvas paintImage: NormalCursor at: bounds topLeft]
		ifFalse: [aCanvas paintImage: self temporaryCursor at: bounds topLeft].
	self hasUserInformation 
		ifTrue: 
			[aCanvas 
				drawString: self userInitials
				at: self cursorBounds topRight + (0 @ 4)
				font: nil
				color: color.
			(userPic := self userPicture) ifNotNil: 
					[aCanvas paintImage: userPic at: self cursorBounds topRight + (0 @ 24)]]! !

!HandMorph methodsFor: '*Rest-Client' stamp: 'SR 7/22/2009 16:24'!
ensureLoginAndPasswordExistAndDo: aBlock

	(((self hasProperty: #restLogin) and: [self hasProperty: #restPassword]) or: [self retrieveLoginAndPassword])
		ifTrue: [aBlock value: self login value: self password]! !

!HandMorph methodsFor: '*Rest-Client' stamp: 'SR 7/22/2009 16:21'!
login

	^ self valueOfProperty: #restLogin! !

!HandMorph methodsFor: '*Rest-Client' stamp: 'SR 7/22/2009 16:21'!
login: login password: password

	self setProperty: #restLogin toValue: login.
	self setProperty: #restPassword toValue: password.! !

!HandMorph methodsFor: '*Rest-Client' stamp: 'SR 7/22/2009 16:09'!
password

	^ self valueOfProperty: #restPassword! !

!HandMorph methodsFor: '*Rest-Client' stamp: 'bs 7/1/2009 10:21'!
resetLoginAndPassword

	self removeProperty: #restLogin.
	self removeProperty: #restPassword.! !

!HandMorph methodsFor: '*Rest-Client' stamp: 'SR 7/22/2009 16:21'!
retrieveLoginAndPassword

	| dialog |
	dialog := RestLoginDialog new ownerHand: self.
	World openModal: dialog.
	^ dialog result! !

!Object methodsFor: '*rest-client' stamp: 'cs 3/17/2009 18:22'!
isRestProxy

	^ false! !

Object subclass: #RestLoginModelForHand
	instanceVariableNames: 'login password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestLoginModelForHand methodsFor: 'initialization' stamp: 'SR 7/1/2009 12:38'!
initialize

	super initialize.
	self
		login: '';
		password: ''.! !

!RestLoginModelForHand methodsFor: 'as yet unclassified' stamp: 'SR 7/1/2009 12:50'!
loadDataFromHand: aHand

	self
		login: aHand login;
		password: aHand password.! !

!RestLoginModelForHand methodsFor: 'accessing' stamp: 'SR 7/1/2009 12:34'!
login

	^ login! !

!RestLoginModelForHand methodsFor: 'accessing' stamp: 'SR 7/1/2009 12:50'!
login: anObject

	login := anObject.
	self changed: #login.! !

!RestLoginModelForHand methodsFor: 'accessing' stamp: 'SR 7/1/2009 12:34'!
password

	^ password! !

!RestLoginModelForHand methodsFor: 'accessing' stamp: 'SR 7/1/2009 12:50'!
password: anObject

	password := anObject.
	self changed: #password.! !

!RestLoginModelForHand methodsFor: 'as yet unclassified' stamp: 'SR 7/1/2009 13:54'!
storeDataToHand: aHand

	aHand login: self login password: self password! !

!RestLoginModelForHand methodsFor: 'as yet unclassified' stamp: 'SR 7/22/2009 12:20'!
validLoginAndPassword

	^ true! !

Object subclass: #RestPredictionState
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Prefetch'!

RestPredictionState subclass: #RestCollectionPredictionState
	instanceVariableNames: 'collection count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Prefetch'!

!RestCollectionPredictionState class methodsFor: 'instance creation' stamp: 'cs 6/9/2009 15:48'!
newFrom: aProxyCollection

	^ self new collection: aProxyCollection! !

!RestCollectionPredictionState class methodsFor: 'constants' stamp: 'cs 6/12/2009 15:56'!
predictionThreshold

	^ 1! !

!RestCollectionPredictionState methodsFor: 'accessing' stamp: 'cs 6/9/2009 15:48'!
collection

	^ collection! !

!RestCollectionPredictionState methodsFor: 'accessing' stamp: 'cs 6/9/2009 15:48'!
collection: anObject

	collection := anObject.! !

!RestCollectionPredictionState methodsFor: 'accessing' stamp: 'cs 6/11/2009 14:24'!
count

	^ count! !

!RestCollectionPredictionState methodsFor: 'accessing' stamp: 'cs 6/11/2009 14:24'!
count: anObject

	count := anObject.! !

!RestCollectionPredictionState methodsFor: 'monitoring' stamp: 'cs 6/12/2009 14:40'!
incrementCount

	self count: self count + 1.
	self count = self class predictionThreshold
		ifTrue: [self prefetch: self collection. ^ false].
	^ true! !

!RestCollectionPredictionState methodsFor: 'initialization' stamp: 'cs 6/11/2009 14:24'!
initialize

	super initialize.
	self count: 0.! !

!RestCollectionPredictionState methodsFor: 'monitoring' stamp: 'cs 6/11/2009 14:28'!
monitor: aProxy 

	((self collection includes: aProxy)
		and: [self incrementCount])
		ifFalse: [self fallback: aProxy].! !

RestPredictionState subclass: #RestObjectPredictionState
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Prefetch'!

!RestObjectPredictionState class methodsFor: 'instance creation' stamp: 'cs 6/9/2009 15:48'!
newFrom: aProxyCollection

	^ self new object: aProxyCollection! !

!RestObjectPredictionState methodsFor: 'monitoring' stamp: 'cs 6/11/2009 14:25'!
monitor: aProxy
	
	((self object properties select: [:value | value isRestProxy])	includes: aProxy)
		ifTrue: [self goto:
			(RestObjectPropertyPredictionState new
				object: self object;
				property: aProxy)]
		ifFalse: [self fallback: aProxy].! !

!RestObjectPredictionState methodsFor: 'accessing' stamp: 'cs 6/9/2009 15:48'!
object

	^ object! !

!RestObjectPredictionState methodsFor: 'accessing' stamp: 'cs 6/9/2009 15:48'!
object: anObject

	object := anObject.! !

RestPredictionState subclass: #RestObjectPropertyPredictionState
	instanceVariableNames: 'object property'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Prefetch'!

!RestObjectPropertyPredictionState methodsFor: 'monitoring' stamp: 'cs 6/12/2009 14:39'!
monitor: aProxy

	((self object properties select: [:value | value isRestProxy])	includes: aProxy)
		ifTrue: [self prefetch: self object]
		ifFalse:
			[self fallback: self property.
			self repository monitor: aProxy].! !

!RestObjectPropertyPredictionState methodsFor: 'accessing' stamp: 'cs 6/11/2009 11:57'!
object

	^ object! !

!RestObjectPropertyPredictionState methodsFor: 'accessing' stamp: 'cs 6/11/2009 11:57'!
object: anObject

	object := anObject.! !

!RestObjectPropertyPredictionState methodsFor: 'accessing' stamp: 'cs 6/10/2009 11:00'!
property

	^ property! !

!RestObjectPropertyPredictionState methodsFor: 'accessing' stamp: 'cs 6/10/2009 11:00'!
property: anObject

	property := anObject.! !

!RestPredictionState methodsFor: 'changing' stamp: 'cs 6/11/2009 12:08'!
fallback: aProxy

	self goto: RestStartPredictionState new.
	self repository monitor: aProxy.! !

!RestPredictionState methodsFor: 'changing' stamp: 'cs 6/9/2009 15:40'!
goto: aState

	self repository monitoringState: aState.
	aState repository: self repository.! !

!RestPredictionState methodsFor: 'monitoring' stamp: 'cs 6/9/2009 15:37'!
monitor: aProxy

	self subclassResponsibility.! !

!RestPredictionState methodsFor: 'prefetching' stamp: 'cs 6/12/2009 14:39'!
prefetch: aRestProxy
		
	self repository prefetch: aRestProxy.! !

!RestPredictionState methodsFor: 'accessing' stamp: 'cs 6/9/2009 15:35'!
repository

	^ repository! !

!RestPredictionState methodsFor: 'accessing' stamp: 'cs 6/9/2009 15:35'!
repository: anObject

	repository := anObject.! !

RestPredictionState subclass: #RestStartPredictionState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Prefetch'!

!RestStartPredictionState methodsFor: 'monitoring' stamp: 'cs 6/9/2009 18:10'!
monitor: aProxy

	aProxy isCollection 
		ifTrue: [self goto: (RestCollectionPredictionState newFrom: aProxy)]
		ifFalse: [self goto: (RestObjectPredictionState newFrom: aProxy)].! !

Object subclass: #RestProxyObject
	instanceVariableNames: 'properties socket restUri repository deferCommit changedWhileDefered'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyObject commentStamp: 'mt 3/2/2009 16:14' prior: 0!
I am a proxy object. I store a reference to the real object in form of a url (e.g. http://bp2008h1srv:8082/projects/5/tasks/3). I behave similar to the real object because I understand its accessors.!

RestProxyObject subclass: #RestPrefetchProxyObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Prefetch'!

!RestPrefetchProxyObject methodsFor: 'updating' stamp: 'cs 6/11/2009 14:58'!
reload

	self repository monitor: self.
	self isLoaded ifFalse: [super reload].! !

!RestPrefetchProxyObject methodsFor: 'updating' stamp: 'cs 6/11/2009 14:58'!
reloadNeeded

	self isLoaded ifTrue: [super reload].! !

!RestProxyObject class methodsFor: 'instance creation' stamp: 'mt 3/31/2009 15:39'!
fromPath: aPathString 

	^ self new restUri: aPathString! !

!RestProxyObject class methodsFor: 'as yet unclassified' stamp: 'SR 7/22/2009 22:52'!
socketClass

	^ (ProcessBrowser isUIProcess: Processor activeProcess)
		ifTrue: [ActiveHandRestSocket]
		ifFalse: [RestSocket]! !

!RestProxyObject class methodsFor: 'helping' stamp: 'topa 3/9/2009 16:57'!
toProperty: aSymbol

 	| selector |
	selector := aSymbol withFirstCharacterDownshifted.
	^ (selector endsWith: 'y') 
		ifTrue: [selector allButLast, 'ies']
		ifFalse: [selector, 's']
	! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mt 3/31/2009 15:38'!
baseUri

	^ self repository url asString! !

!RestProxyObject methodsFor: 'accessing' stamp: 'SR 4/16/2009 22:59'!
changedWhileDefered

	^ changedWhileDefered! !

!RestProxyObject methodsFor: 'accessing' stamp: 'SR 4/16/2009 22:59'!
changedWhileDefered: anObject

	changedWhileDefered := anObject! !

!RestProxyObject methodsFor: 'updating' stamp: 'SR 4/16/2009 22:59'!
commit

	deferCommit ifFalse: [
		self pvtPutContent: (Dictionary newFrom:
			(self properties associationsSelect:
				[:assoc | (self changedWhileDefered includes: assoc key) and: [(assoc key startsWith: 'col ')  not]]))].! !

!RestProxyObject methodsFor: 'updating' stamp: 'SR 4/6/2009 17:48'!
commit: key 

	deferCommit
		ifFalse: [self pvtPutContent: (Dictionary with: key -> (self properties at: key))]
		ifTrue: [self changedWhileDefered add: key]! !

!RestProxyObject methodsFor: 'updating' stamp: 'SR 4/6/2009 17:49'!
deferCommit: aBlock

	deferCommit := true.
	self changedWhileDefered: Set new.
	^ aBlock ensure: [deferCommit := false. self commit]! !

!RestProxyObject methodsFor: 'deleting' stamp: 'cs 4/22/2009 18:33'!
delete

	self repository deleteObject: self restUri.
	self reloadObsoleteReferences.
	self changed: #deleted.! !

!RestProxyObject methodsFor: 'error handling' stamp: 'SR 4/16/2009 22:59'!
doesNotUnderstand: aMessage

	"handle getters"
	(self includesProperty: aMessage selector)
		ifTrue: [^ self propertiesAt: aMessage selector].

	"handle add"
	((aMessage selector startsWith: 'add') and: [aMessage selector endsWith: ':'])
		ifTrue: [^ self handleAddMessage: aMessage].
	
	"handle remove"
	((aMessage selector startsWith: 'remove') and: [aMessage selector endsWith: ':'])
		ifTrue: [^ self handleRemoveMessage: aMessage].

	" handle setters"
	((aMessage selector endsWith: ':') and:
		[self includesProperty: aMessage selector allButLast])
			ifTrue: [^ self set: aMessage selector allButLast value: aMessage arguments first].

	"handle unknown"
	super doesNotUnderstand: aMessage.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'SR 4/16/2009 23:24'!
handleAddMessage: aMessage
	
	| target property targetSelector result |
	target :=  aMessage selector allButFirst: 3.
	
	targetSelector := (target startsWith: 'New') 
		ifTrue: [target :=  target allButFirst: 3. #addNew:]
		ifFalse:  [#add:].
	
	property := self propertiesAt: (self class toProperty: target allButLast).		
	result := property perform: targetSelector withArguments: aMessage arguments.
	
	self changed: (self class toProperty: target allButLast) asSymbol.
	self changed: ((self class toProperty: target allButLast), 'WithDeleted') asSymbol.	
	^ result
! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 4/22/2009 18:32'!
handleRemoveMessage: aMessage
	
	| target property result |
	target :=  aMessage selector allButFirst: 6.
	
	property := self propertiesAt: (self class toProperty: target allButLast).		
	result := property perform: #remove:ifAbsent: with: aMessage arguments first with: nil.
	
	self changed: (self class toProperty: target allButLast) asSymbol.
	self changed: ((self class toProperty: target allButLast), 'WithDeleted') asSymbol.
	^ result! !

!RestProxyObject methodsFor: 'helping' stamp: 'mt 4/21/2009 16:34'!
includesProperty: aSelector
	
	^ (self propertyKeyFor: aSelector) notNil! !

!RestProxyObject methodsFor: 'initialization' stamp: 'SR 4/6/2009 17:47'!
initialize

	super initialize.
	deferCommit := false.
	changedWhileDefered := Set new.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 6/11/2009 14:50'!
isLoaded

	^ properties notNil! !

!RestProxyObject methodsFor: 'testing' stamp: 'cs 3/17/2009 18:21'!
isRestProxy

	^ true! !

!RestProxyObject methodsFor: 'updating' stamp: 'mt 3/16/2009 12:26'!
jsonWriteOn: aStream

	self restUri jsonWriteOn: aStream.! !

!RestProxyObject methodsFor: 'helping' stamp: 'cs 5/13/2009 15:34'!
keyToName: key
	
	((key startsWith: 'ref ') or:
		[key startsWith: 'col ']) ifTrue: [
		^ (key allButFirst: 4) asSymbol].
	^ key asSymbol! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 4/26/2009 18:20'!
moved

	self
		reloadObsoleteReferences;
		reloadMissingReferences;
		changed: #moved.! !

!RestProxyObject methodsFor: 'helping' stamp: 'mig 4/1/2009 16:57'!
parseJsonKey: key value: value

	(key startsWith: 'col ') ifTrue: [
		^ key -> ((self repository getCollection:
			('{1}/{2}' format: {self restUri. key allButFirst: 4}))
				fill: value)].
	
	(key startsWith: 'ref ') ifTrue: [
		value
			ifNil: [^ key -> nil]
			ifNotNil: [^ key -> (self repository getObject: value)]].

	^ key -> value
! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 17:40'!
properties

	^ properties ifNil: [self reload. properties]! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 5/13/2009 15:32'!
properties: aDictionary
	
	| oldProperties |
	oldProperties := properties.
	properties := aDictionary.
	
	self properties keysAndValuesDo: [:key :value |
		(oldProperties notNil and: [(oldProperties at: key ifAbsent: []) ~= value])
			ifTrue: [self changed: (self keyToName: key)]].
	self restUri: (self properties at: 'restUri').! !

!RestProxyObject methodsFor: 'helping' stamp: 'SR 7/22/2009 22:08'!
propertiesAt: aSelector
	
	| result |
	self flag: #MayOccurAgain.
	result := self properties at: aSelector ifAbsent:
		[self properties at: 'ref ', aSelector ifAbsent:
			[self properties at: 'col ', aSelector]].
	^ result "isString 
		ifTrue: [result utf8ToSqueak]
		ifFalse: [result]"! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 4/27/2009 17:57'!
propertiesFromDictionary: aDictionary

	self properties:
		(aDictionary keysAndValuesCollect:
			[:key :value | self parseJsonKey: key value: value]).! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 4/27/2009 17:57'!
propertiesFromResponse: responseString

	self propertiesFromDictionary: (Json readFrom: responseString readStream).
! !

!RestProxyObject methodsFor: 'helping' stamp: 'mt 4/21/2009 16:04'!
propertyKeyFor: aSelector
	
	^ (self properties includesKey: aSelector)
		ifTrue: [aSelector]
		ifFalse: [(self properties includesKey: 'ref ', aSelector)
			ifTrue: ['ref ', aSelector]
			ifFalse: [(self properties includesKey: 'col ', aSelector)
				ifTrue: ['col ', aSelector]]]! !

!RestProxyObject methodsFor: 'updating' stamp: 'SR 7/22/2009 22:11'!
pvtPutContent: aDictionary 
	
	| response |
	response := self class socketClass 
		putDocumentFromUrl: (self baseUri , self restUri) asUrl
		request: ''
		content: (Json render: aDictionary).
		
	response isOk
		ifTrue: [self restUri: response content]
		ifFalse: [RestProxyRepository errorForCode: response responseCode].! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 4/22/2009 17:41'!
referencedProxies

	^ properties
		ifNil: [#()]
		ifNotNil: [self referencedProxiesOf: self properties]! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 6/11/2009 15:00'!
referencedProxiesOf: aDictionary 

	| result |
	result := Set new.
	aDictionary ifNil: [^ result].
	aDictionary values 
		select: [:value | value isRestProxy]
		thenDo: [:proxy | result add: proxy.
			(proxy isCollection and: [proxy isLoaded])
				ifTrue: [result addAll: proxy]].
	^ result! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 4/25/2009 16:48'!
referencesProxy: anRestProxy

	^ self ~= anRestProxy and: [self referencedProxies includes: anRestProxy]! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 4/22/2009 17:31'!
referencingProxies

	^ self repository dependentsOf: self! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 4/27/2009 17:50'!
reload

	self reloadDepth: 1.! !

!RestProxyObject methodsFor: 'updating' stamp: 'SR 7/22/2009 22:11'!
reloadDepth: aNumber

	| response |
	response := self class socketClass get: ('{1}{2}{3}' format: {
			self baseUri. self restUri.
			aNumber ~= 1 ifTrue: ['?depth=', aNumber asString] ifFalse: ['']}).
	response isOk
		ifTrue: [self propertiesFromResponse: response content. ^ self].
	response isNotFound
		ifTrue: [self removeFromCache; changed: #deleted. ^ self].

	RestProxyRepository errorForCode: response responseCode.! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 4/27/2009 10:18'!
reloadMissingReferences

	(self referencedProxies
		reject: [:proxy | proxy isCollection])
		reject: [:proxy | proxy referencesProxy: self]
		thenDo: [:proxy | proxy reloadNeededWithCollections]! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 4/22/2009 18:10'!
reloadNeeded

	properties ifNotNil: [self reload].! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 6/11/2009 15:01'!
reloadNeededWithCollections

	self reloadNeeded.
	self isLoaded ifTrue: 
		[self properties values 
			select: [:value | value isRestProxy & value isCollection]
			thenDo: [:proxyCollection | proxyCollection reloadNeeded]].! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 4/26/2009 18:17'!
reloadObsoleteReferences

	self referencingProxies asOrderedCollection
		removeAllFoundIn: self referencedProxies;
		do: [:proxy | proxy reloadNeeded].! !

!RestProxyObject methodsFor: 'deleting' stamp: 'mig 4/3/2009 12:00'!
removeFromCache

	^ self repository removeFromCache: self restUri
! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mt 3/16/2009 11:21'!
repository

	^ repository! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mt 3/16/2009 11:22'!
repository: anObject

	repository := anObject.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 3/17/2009 17:57'!
restUri

	^ restUri! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mfp 4/29/2009 15:37'!
restUri: anObject

	(self repository notNil and: [restUri ~= anObject]) ifTrue:
		[restUri notNil ifTrue: [self repository cache removeKey: restUri ifAbsent: []].
		anObject notNil ifTrue: [self repository cache at: anObject put: self].
		(restUri notNil and: [anObject notNil]) ifTrue: [self moved]].
	restUri := anObject.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mig 4/21/2009 10:53'!
set: property value: value

	self properties
		at: (self propertyKeyFor: property)
		put: value.

	self 
		commit: (self propertyKeyFor: property);
		changed: property.! !

Object subclass: #RestProxyRepository
	instanceVariableNames: 'url cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyRepository commentStamp: 'mt 3/2/2009 16:39' prior: 0!
I provide convenient access to a rest server by creating rest proxy objects.!

RestProxyRepository subclass: #RestPrefetchProxyRepository
	instanceVariableNames: 'monitoringState rules'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Prefetch'!

!RestPrefetchProxyRepository class methodsFor: 'helping' stamp: 'cs 6/9/2009 14:40'!
proxyCollectionClass

	^ RestPrefetchProxyCollection! !

!RestPrefetchProxyRepository class methodsFor: 'helping' stamp: 'cs 6/9/2009 14:40'!
proxyObjectClass

	^ RestPrefetchProxyObject! !

!RestPrefetchProxyRepository methodsFor: 'initialization' stamp: 'cs 6/11/2009 15:06'!
initialize

	super initialize.
	self monitoringState: (RestStartPredictionState new repository: self).
	self rules: OrderedCollection new.! !

!RestPrefetchProxyRepository methodsFor: 'monitoring' stamp: 'cs 6/12/2009 13:37'!
monitor: aProxy 

	self monitoringState monitor: aProxy.! !

!RestPrefetchProxyRepository methodsFor: 'accessing' stamp: 'cs 6/12/2009 16:04'!
monitoringState

	monitoringState ifNil: [self monitoringState:
		(RestStartPredictionState new repository: self)].
	^ monitoringState! !

!RestPrefetchProxyRepository methodsFor: 'accessing' stamp: 'cs 6/9/2009 15:38'!
monitoringState: anObject

	monitoringState := anObject.! !

!RestPrefetchProxyRepository methodsFor: 'prefetching' stamp: 'cs 6/12/2009 15:47'!
prefetch: aProxy

	self prefetch: aProxy depth: 2.! !

!RestPrefetchProxyRepository methodsFor: 'prefetching' stamp: 'cs 6/12/2009 15:26'!
prefetch: aProxy depth: aNumber

	aProxy reloadDepth: aNumber.
	self rules add: aProxy -> aNumber.! !

!RestPrefetchProxyRepository methodsFor: 'accessing' stamp: 'cs 6/12/2009 16:04'!
rules

	rules ifNil: [self rules: OrderedCollection new].
	^ rules! !

!RestPrefetchProxyRepository methodsFor: 'accessing' stamp: 'cs 6/11/2009 15:06'!
rules: anObject

	rules := anObject.! !

!RestProxyRepository class methodsFor: 'helping' stamp: 'SR 7/23/2009 10:16'!
errorForCode: httpErrorCode

	httpErrorCode
		caseOf: {
			[404] -> [RestEntityNotFoundError signal: 'REST-Entity was not found on server.'].
	     	[401] -> [].}
		otherwise: [UITheme current
			alertIn: World
			text: 'Error retrieving contents from server. HTTP response code was ', httpErrorCode asString ,'. Please report this error to the developers!!!!'
			title: 'Network Error'.
			RestUnexpectedResponseError signal].! !

!RestProxyRepository class methodsFor: 'instance creation' stamp: 'topa 3/5/2009 15:30'!
fromString: aServerString
	"e.g. http://bp2008h1srv:8082/"

	^ self new url: aServerString asUrl! !

!RestProxyRepository class methodsFor: 'helping' stamp: 'mig 4/3/2009 15:33'!
proxyCollectionClass

	^ RestProxyCollection! !

!RestProxyRepository class methodsFor: 'helping' stamp: 'mig 4/3/2009 15:26'!
proxyObjectClass

	^ RestProxyObject! !

!RestProxyRepository class methodsFor: 'as yet unclassified' stamp: 'SR 7/1/2009 14:07'!
url: aString

	^ self url: aString username: nil password: nil! !

!RestProxyRepository class methodsFor: 'instance creation' stamp: 'mt 3/31/2009 15:34'!
url: url username: userName password: password
	"self password ifEmpty: [^ 'http://{1}@{2}:{3}/' format: {self login encodeForHTTP. self host. self port}]." "not allowed."
	| myUrl myStream |
	myUrl := url asUrl.
	
	myStream := WriteStream on: ''.
	myStream nextPutAll: myUrl schemeName.
	myStream nextPutAll: '://'.
	userName ifNotNil: [
		myStream nextPutAll: userName encodeForHTTP.
		password ifNotNil: [
			myStream nextPutAll: ':'.
			myStream nextPutAll: password encodeForHTTP ].
		myStream nextPutAll: '@' ].
	myStream nextPutAll: myUrl authority.
	myUrl port ifNotNil: [myStream nextPut: $:; print: myUrl port].
	myUrl path do: [ :pathElem |
		myStream nextPut: $/.
		myStream nextPutAll: pathElem encodeForHTTP. ].
	myUrl query isNil ifFalse: [ 
		myStream nextPut: $?.
		myStream nextPutAll: self query. ].
	myUrl fragment isNil ifFalse: [
		myStream nextPut: $#.
		myStream nextPutAll: self fragment encodeForHTTP. ].
	^ myStream contents! !

!RestProxyRepository methodsFor: 'instance creation' stamp: 'SR 7/22/2009 22:13'!
addObject: path with: anObject 

	| response |
	response := RestProxyObject socketClass 
				postDocumentFromUrl: (self url asString, path) asUrl
				request: ''
				content: (Json render: anObject).
	^ response isOk
		ifTrue: [self getObject: response content]
		ifFalse: [RestProxyRepository errorForCode: response responseCode]! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'cs 3/17/2009 16:07'!
cache

	^ cache! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'cs 3/17/2009 16:07'!
cache: anObject

	cache := anObject.! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'cs 3/18/2009 12:13'!
cacheAt: restUri ifAbsent: aBlock

	^ (cache includesKey: restUri)
		ifFalse: aBlock
		ifTrue: [(cache at: restUri) ifNil: aBlock]! !

!RestProxyRepository methodsFor: 'private' stamp: 'SR 7/22/2009 22:13'!
deleteObject: restUri

	| response |
	[response := RestProxyObject socketClass
		deleteDocumentFromUrl: (self url asString, restUri) asUrl
		request: ''.
	response isOk | response isNotFound "Might be deleted already."
		ifFalse: [RestProxyRepository errorForCode: response responseCode]
	] ensure: [self removeFromCache: restUri].! !

!RestProxyRepository methodsFor: 'updating' stamp: 'cs 4/22/2009 17:29'!
dependentsOf: aRestProxy

	^ (self cache associations 
		select: [:assoc | assoc value notNil]
		thenCollect: [:assoc | assoc value])
		select: [:proxy | proxy referencesProxy: aRestProxy]! !

!RestProxyRepository methodsFor: 'updating' stamp: 'cs 4/22/2009 18:44'!
dependentsOfUri: restUri

	^ (self cache associations 
		select: [:assoc | assoc value notNil]
		thenCollect: [:assoc | assoc value])
		select: [:proxy | proxy referencedProxies anySatisfy: [:proxy2 | proxy2 restUri = restUri]]! !

!RestProxyRepository methodsFor: 'instance creation' stamp: 'mig 4/3/2009 15:33'!
getCollection: path 
	"Get list of cn's and request their Objects into a Collection"
	
	^ self
		cacheAt: path 
		ifAbsent: [self cache at: path put: (
			(self class proxyCollectionClass fromPath: path) repository: self)]! !

!RestProxyRepository methodsFor: 'instance creation' stamp: 'mig 3/13/2009 15:33'!
getCollection: path withContents: anURICollection
	"build a proxy collection for the given URIs"
	
	^ (self getCollection: path) fill: anURICollection! !

!RestProxyRepository methodsFor: 'instance creation' stamp: 'cs 4/27/2009 17:56'!
getObject: path 

	path isDictionary ifTrue: [
		^ (self getObject: (path at: 'restUri')) propertiesFromDictionary: path].
	
	^ self
		cacheAt: path
		ifAbsent: [self cache at: path put: (
			(self class proxyObjectClass fromPath: path)
			repository: self)]! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'cs 3/18/2009 15:03'!
includesObject: anObject

	^ self cache values includes: anObject! !

!RestProxyRepository methodsFor: 'initialization' stamp: 'SR 3/19/2009 10:56'!
initialize

	super initialize.
	self cache: Dictionary new.! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'SR 7/1/2009 19:45'!
login

	^ self url username
! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'SR 7/1/2009 19:45'!
login: aString

	self url:
		(self class
			url: self url
			username: aString
			password: self password)
! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'SR 7/1/2009 19:45'!
password

	^ self url password
! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'SR 7/1/2009 19:45'!
password: aString

	self url:
		(self class
			url: self url
			username: self login
			password: aString)! !

!RestProxyRepository methodsFor: 'private' stamp: 'cs 4/22/2009 18:45'!
removeFromCache: restUri

	self cache keysAndValuesRemove: [:key :value | key startsWith: restUri].
	(self dependentsOfUri: restUri) do: [:proxy | proxy reload].! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'topa 3/5/2009 15:30'!
url

	^ url! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'topa 3/5/2009 15:30'!
url: anObject

	url := anObject.! !

Object subclass: #RestResponse
	instanceVariableNames: 'content headers responseCode url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestResponse class methodsFor: 'instance creation' stamp: 'mfp 2/20/2009 16:05'!
responseCode: aNumberCode headers: aDictionary content: aString url: aUrl

	^ self new
		responseCode: aNumberCode asNumber;
		headers: aDictionary;
		content: aString;
		url: aUrl! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
content

	^ content! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
content: anObject

	content := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:49'!
getHeader: name 

	^ self getHeader: name default: nil! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
getHeader: name  default: defaultValue

	^ self headers at: name  ifAbsent: [defaultValue]! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
headers

	^ headers! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
headers: anObject

	headers := anObject.! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isBadRequest


	^ self responseCode = 400! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isFound


	^ self responseCode = 302! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isInternalServerError


	^ self responseCode = 500! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isMovedPermanently


	^ self responseCode = 301! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isNotFound


	^ self responseCode = 404! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isNotImplemented


	^ self responseCode = 501! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isNotModified


	^ self responseCode = 304! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:21'!
isOk


	^ self responseCode = 200! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:21'!
isRedirectLink


	^ self responseCode = 302! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:21'!
isSeeOther


	^ self responseCode = 303! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:21'!
isUnauthorized


	^ self responseCode = 401! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
responseCode

	^ responseCode! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
responseCode: anObject

	responseCode := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
url

	^ url! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
url: anObject

	url := anObject.! !

Object subclass: #RestSocket
	instanceVariableNames: 'conn headers responseCode request content url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

RestSocket class
	instanceVariableNames: 'counter'!

RestSocket class
	instanceVariableNames: 'counter'!

RestSocket subclass: #ActiveHandRestSocket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!ActiveHandRestSocket class methodsFor: 'as yet unclassified' stamp: 'SR 7/23/2009 11:27'!
doHTTPRequestFromUrl: url verb: aVerb request: requestString content: contentString

	| response |
	3 timesRepeat: [
		response := super doHTTPRequestFromUrl: url verb: aVerb request: requestString content: contentString.
		response responseCode = 401 ifFalse: [^ response].
		ActiveHand resetLoginAndPassword].
	^ response! !

!ActiveHandRestSocket methodsFor: 'as yet unclassified' stamp: 'SR 7/22/2009 22:05'!
userNamePasswordIfExists
	
	ActiveHand ensureLoginAndPasswordExistAndDo: [:login :password | ^ 
		'Authorization: Basic ',
			((Base64MimeConverter mimeEncode: 
			(login, ':', password) readStream) upToEnd select: [:c | c ~= Character cr]), 
			String crlf].
	^ ''! !

!RestSocket class methodsFor: 'accessing' stamp: 'cs 6/11/2009 14:18'!
counter

	^ counter ifNil: [counter := 0].! !

!RestSocket class methodsFor: 'private' stamp: 'topa 3/3/2009 21:40'!
defaultPortForSchema: aSchema
	"default port to connect on"

	aSchema = 'http' ifTrue: [^ 80].
	aSchema = 'https' ifTrue: [^ 443].! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:40'!
deleteDocumentFromUrl: url request: requestString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'DELETE'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'requesting' stamp: 'SR 7/23/2009 11:25'!
doHTTPRequestFromUrl: url verb: aVerb request: requestString content: contentString

	| serverName sock |
	self flag: #NeedsRefactoring.
	self incrementCounter.
	Socket initializeNetwork.
	3 timesRepeat: [
		sock := self new 
			url: url;
			request: requestString;
			content: contentString.
		(sock connectAndWaitUntil: (Time millisecondClockValue + 10000 truncated))
			ifTrue: [^ sock doVerb: aVerb]].
	serverName := url authority.
	Socket deadServer: serverName.
	sock destroy.
	UITheme current 
		alertIn: World
		text: 'Server' , serverName , ' is not responding'
		title: 'Network Error'.
	NetworkError new signal: 'Server ' , serverName , ' is not responding'.
! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
expandUrl: newUrl ip: byteArrayIP port: portNum

	self flag: #SplitUp; flag: #NeedsRefactoring.
	(newUrl beginsWith: '../') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: (newUrl allButFirst: 2)]].
	(newUrl beginsWith: '/') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: newUrl]].
	^ newUrl
! !

!RestSocket class methodsFor: 'requesting' stamp: 'SR 2/26/2009 14:59'!
get: aUriString
	"Send an HTTP GET request."

	^ RestSocket 
		getDocumentFromUrl: aUriString asUrl "for Socket compatibility"
		request: ''! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
getDocumentFromUrl: url request: requestString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'GET'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'private' stamp: 'cs 6/11/2009 14:19'!
incrementCounter

	counter := self counter + 1.! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
postDocumentFromUrl: url request: requestString content: contentString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'POST'
		request: requestString
		content: contentString! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
putDocumentFromUrl: url request: requestString content: contentString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'PUT'
		request: requestString
		content: contentString! !

!RestSocket class methodsFor: 'accessing' stamp: 'cs 6/11/2009 14:19'!
resetCounter

	counter := 0.! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
conn

	^ conn! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
conn: anObject

	conn := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
connect
	| serverAddr |
	serverAddr := (NetNameResolver addressForName: self serverName timeout: 20)
		ifNil: [self error: 'Could not resolve the server named: ', self serverName].
		
	self connectTo: serverAddr port: self port.! !

!RestSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:41'!
connectAndWaitUntil: aTimeout

	^ self connect; waitForConnectionUntil: aTimeout! !

!RestSocket methodsFor: 'api' stamp: 'mfp 4/28/2009 16:48'!
connectTo: addr port: aPort

	self conn: (self usesSSL
			ifTrue: [SSLSocket newConnectionAddress: addr port: aPort version: SSLProtocolVersion tls10]
			ifFalse: [[Socket new connectTo: addr port: aPort] 
				on: ConnectionTimedOut do: 
					[UITheme current
						alertIn: World
						text: 'Connection to server lost!! Please close the application in order to prevent further loss of data!!'
						title: 'Network Error'.
					RestConnectionLostError signal]]).
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:32'!
content

	^ content! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:32'!
content: anObject

	content := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
destroy

	self conn close.! !

!RestSocket methodsFor: 'api' stamp: 'cs 5/2/2009 12:01'!
doVerb: aHTTPVerb

	| cmd stream |
	cmd := aHTTPVerb,' ', self url fullPath, ' HTTP/1.0', String crlf, 
				'Host: ', self serverName , ':', self port printString, String crlf, 
				self userNamePasswordIfExists,
				(self request ifEmpty: [''] ifNotEmpty: [self request, String crlf]).
	self content ifNotNil: [cmd := cmd, 
		'Content-type: application/json', String crlf,
		'Content-length: ', self content size printString , String crlf].
	self sendCommand: cmd.

	"headers are sent."	
	self content ifNotNil: [self sendCommand: self content].
	stream := self getData readStream.
	self headersFrom: (stream upToAll: String crlfcrlf). "parses header, too"
	
	^ ({'301'. '302'} includes: self responseCode) 
		ifTrue: [" redirect"
			self class 
				doHTTPRequestFromUrl: (self url newFromRelativeText: (self getHeader: 'location'))
				verb: aHTTPVerb
				request: self request
				content: self content
		] ifFalse: [" no redirect"
			RestResponse 
				responseCode: self responseCode  
				headers: self headers
				content: (self getRestOfData: stream upToEnd)
				url: self url
			]
! !

!RestSocket methodsFor: 'api' stamp: 'mig 4/29/2009 00:07'!
getData

	| data |
	data := self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveData].
	[self conn isConnected and: [(data asString findString: String crlfcrlf) == 0]]
		whileTrue: [
			data := data, (self usesSSL
				ifTrue: [self conn receive]
				ifFalse: [self conn receiveData])].
	^ data asString! !

!RestSocket methodsFor: 'api' stamp: 'mig 4/28/2009 22:50'!
getHeader: name 

	^ self getHeader: name default: nil! !

!RestSocket methodsFor: 'api' stamp: 'mig 4/28/2009 22:50'!
getHeader: name  default: defaultValue

	^ self headers at: name ifAbsent: [defaultValue]! !

!RestSocket methodsFor: 'api' stamp: 'mig 4/29/2009 11:33'!
getRestOfData: begin

	| data length |
	length := self getHeader: 'content-length'.
	length ifNotNil: [length := length asNumber].

	data := begin, (self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveData]).
	[self conn isConnected and: [length isNil or: [data size < length]]]
		whileTrue: [
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveData]).].
	^ data asString! !

!RestSocket methodsFor: 'api' stamp: 'mig 4/29/2009 11:02'!
headerFromLine: aString

	| splitPosition |
	splitPosition := aString indexOf: $:.
	splitPosition > 0 ifTrue: [
		self headers 
			at: (aString copyFrom: 1 to: splitPosition-1) asLowercase 
			put: (aString copyFrom: splitPosition+1 to: aString size) withBlanksTrimmed ].
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:37'!
headers

	^ headers! !

!RestSocket methodsFor: 'api' stamp: 'mig 4/29/2009 11:05'!
headersFrom: headerText

	| lines foldedLines |
	"divide into basic lines"
	lines := headerText findTokens: (String with: Character cr with: Character linefeed).

	responseCode := (lines first findTokens: ' ') second.

	"fold lines that start with spaces into the previous line"
	foldedLines := OrderedCollection new.
	lines allButFirst do: [ :line |
		line first isSeparator
			ifTrue: [ foldedLines atLast: 1 put: (foldedLines last, line) ]
			ifFalse: [ foldedLines add: line ]].

	"make a dictionary mapping headers to header contents"
	headers := Dictionary new.
	foldedLines do: [ :line | self headerFromLine: line ].
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
isConnecting

	^ self usesSSL
		ifTrue: [self conn isConnecting]
		ifFalse: [self conn isWaitingForConnection]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
port

	^ self url port 
		ifNil:  [self class defaultPortForSchema: self url schemeName]
		ifNotNil: [self url port]
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
request

	^ request! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
request: anObject

	request := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:38'!
responseCode

	^ responseCode! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:38'!
sendCommand: cmd
	
	self usesSSL
		ifTrue: [self conn send: cmd, String crlf]
		ifFalse: [self conn sendData: cmd, String crlf]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
serverName

	^ self url authority! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
url

	^ url! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
url: anObject

	url := anObject.! !

!RestSocket methodsFor: 'accessing' stamp: 'mig 5/8/2009 11:02'!
userNamePasswordIfExists

	^ (self url username notNil) & (self url password notNil)
		ifFalse: ['']
		ifTrue: ['Authorization: Basic ',
				((Base64MimeConverter mimeEncode: 
				(self url username unescapePercents, ':', self url password unescapePercents) readStream) upToEnd select: [:c | c ~= Character cr]), 
				String crlf]
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
usesSSL

	^ url schemeName = 'https'! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:39'!
waitForConnectionUntil: deadline
	"Wait up until the given deadline for a connection to be established. 
	Return true if it is established by the deadline, false if not."

	[self isConnecting and: [Time millisecondClockValue < deadline]]
		whileTrue: [(Delay forMilliseconds: 100) wait].
	^ self conn isConnected
! !

!Dictionary methodsFor: '*rest-client' stamp: 'mt 2/16/2009 10:28'!
keysAndValuesCollect: aBlock 
	| newDict |
	newDict := Dictionary new.
	self keysAndValuesDo: [:key :value | newDict add: (aBlock value: key value: value)].
	^ newDict! !

Error subclass: #RestConnectionLostError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

Error subclass: #RestEntityNotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

Error subclass: #RestUnexpectedResponseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!
